/* Industrialio test ring buffer
 *
 * Copyright (c) 2008 Jonathan Cameron
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * Little tool to poke character devices associated with ring buffer and see
 * what events are coming out.
 *
 * Todo: Make this more adapatable - e.g. allow specification of which
 *       ring to poke.
 */
#include <dirent.h>
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>
#include <stdint.h>
#include <linux/types.h>
#include <sys/stat.h>
#include <sys/dir.h>



struct iio_event_data {
	int id;
	__s64 timestamp;
};
#define IIO_EVENT_CODE_RING_50_FULL 200
#define IIO_EVENT_CODE_RING_75_FULL 201
#define IIO_EVENT_CODE_RING_100_FULL 202
/* returns the base device directory of a named sensor */
char *find_device_by_name(const char *name)
{
	const char *iio_dir = "/sys/class/industrialio";
	const struct dirent *ent;
	int cnt;
	FILE *nameFile;
	char thisname[100];
	char temp[100];
	DIR *dp;
	int pos, pos2;
	char *returnstring = NULL;
	struct stat Stat;
	pos = sprintf(temp,"%s", iio_dir);
	pos += sprintf(temp+pos, "/");
	dp = opendir(iio_dir);
	if(dp == NULL) {
		printf("No industrialio devices available");
		return NULL;
	}
	while (ent = readdir(dp), ent != NULL) {
		cnt++;
		/*reject . and .. */
		if (strcmp(ent->d_name,".") != 0 &&
		    strcmp(ent->d_name,"..") != 0)  {
			/*make sure it isn't a trigger!*/
			if(strncmp(ent->d_name,"iio_trigger",10) != 0) {
				/* build full path to new file */
				pos2 = pos + sprintf(temp+pos,"%s/device/", ent->d_name);
				sprintf(temp+pos2,"name");
				nameFile = fopen(temp, "r");
				if (!nameFile) {
					sprintf(temp+pos2,"modalias", ent->d_name);
					nameFile = fopen(temp, "r");
					if(!nameFile) {
						printf("Failed to find a name for device\n");
						return NULL;
					}
				}
				fscanf(nameFile, "%s", thisname);
				if (strcmp(name, thisname) ==0) {
					returnstring = malloc(strlen(temp) + 1);
					sprintf(temp+pos2,"");
					strcpy(returnstring, temp);
//					strcpy(returnstring + strlen(temp) -4, "");
					return returnstring;
				}
				fclose(nameFile);

			}
		}
	}
}

struct industrialio_event_data {
	int id;
	__s64 timestamp;
};

union lis3l02dq_channel {
	char data[2];
	int16_t val;
};

struct lis3l02dq_datum {
	union lis3l02dq_channel el[3];
	__s64 time;
};
int RingLength = 100;
int NumVals = 3;
int main(int argc, char** argv)
{
	int toread, sample = 0;
	uint16_t tempx, tempy, tempz;
	FILE  *sysfsfp, *fp_ev, *ringcont;
	int fp;
	struct lis3l02dq_datum data[1000];
	const char *device_name = "lis3l02dq";
	size_t read_size;
	int i ,j;
	int minor, minor_ev;
	char name_chrdevevt[100];
	char name_chrdevring[100];
	char command[100];
	int device_no;
	char temp[200];
	int pos;
	struct industrialio_event_data dat;

	char *BaseDirectoryName = find_device_by_name(device_name);
	if(BaseDirectoryName == NULL) {
		printf("Failed to find the %s \n", device_name);
		return -1;
	}

	/* Set a sensible ring buffer length */

	sprintf(temp, "%sring_buffer/length", BaseDirectoryName);
	sysfsfp = fopen(temp, "w");
	if (sysfsfp == NULL) {
		printf("failed to open the ring buffer length file at %s\n", temp);
		return -1;
	}
	fprintf(sysfsfp, "%d", RingLength);
	fclose(sysfsfp);
	/* Create relevant device Nodes */
	sprintf(temp, "%sring_buffer0_access_minor", BaseDirectoryName);
	sysfsfp = fopen(temp, "r");
	if (sysfsfp == NULL) {
		printf("failed to open minor stuff %s \n", temp);
		return -1;
	}

	fscanf(sysfsfp,"%d\n", &minor);
	sprintf(name_chrdevring, "/dev/indring%d", minor);

	fclose(sysfsfp);
	sprintf(temp, "%sring_buffer0_ev_minor", BaseDirectoryName);
	sysfsfp = fopen(temp, "r");
	if (sysfsfp == NULL) {
		printf("failed to open minor s3tuff \n");
	}

	fscanf(sysfsfp, "%d\n", &minor_ev);
	fclose(sysfsfp);
	sprintf(name_chrdevevt, "/dev/indringev%d", minor_ev);



	/* Enable the ring buffer */
	{

		sprintf(temp, "%sring_buffer/ring_enable", BaseDirectoryName);
		ringcont = fopen(temp, "w");

		if (ringcont == NULL) {
			printf("Failed to open the ring buffer control file \n");
			return -1;
		};

		fprintf(ringcont, "1");

		fclose(ringcont);
	}
	/* Attempt to open non blocking the access dev */
	fp = open(name_chrdevring, O_RDONLY | O_NONBLOCK);
	if (fp == -1) { /*If it isn't there make the node */
		sprintf(command, "mknod %s c 244 %d; mknod %s c 244 %d; ",
			name_chrdevring, minor, name_chrdevevt, minor_ev);
		system(command);
		sleep(5);
		fp = open(name_chrdevring, O_RDONLY | O_NONBLOCK);
		if (fp == -1) {
			printf("Unable to open %s\n", name_chrdevring);
			return -1;
		}
	}
	/* Attempt to open the event access dev (blocking this time) */
	fp_ev = fopen(name_chrdevevt, "rb");
	if (fp_ev == NULL) {

		printf("bug opening %s\n", name_chrdevevt);
		return -1;
	}

	/* Wait for events 10 times */
	for (j = 0; j < 10000; j++) {
		read_size = fread(&dat, 1, sizeof(struct iio_event_data),
				  fp_ev);
		switch(dat.id) {
		case IIO_EVENT_CODE_RING_100_FULL:
		        toread = RingLength;
			break;
		case IIO_EVENT_CODE_RING_75_FULL:
			toread = RingLength*3/4;
			break;
		case IIO_EVENT_CODE_RING_50_FULL:
			toread = RingLength/2;
			break;
		default:
			printf("Unexpecteded event code\n");
		}
		read_size = read(fp, (char *)(data), toread*sizeof(struct lis3l02dq_datum));
		if (read_size == -EAGAIN)
			printf("nothing available \n");
		for(i = 0; i < (read_size /sizeof(struct lis3l02dq_datum ))*0.1; i++) {
			printf("%d, %d, %d, %d : %lld\n",
			       sample++,
			       data[i].el[0].val,
			       data[i].el[1].val,
			       data[i].el[2].val,
			       data[i].time);
		}

	}
	close(fp);
	fclose(fp_ev);
	/* Stop the ring buffer */
	{

		sprintf(temp, "%sring_buffer/ring_enable", BaseDirectoryName);
		ringcont = fopen(temp, "w");
		if (ringcont == NULL) {
			printf("Failed to open the ring buffer control file \n");
			return -1;
		};
		fprintf(ringcont, "0");

		fclose(ringcont);
	}

	free(BaseDirectoryName);

	return 0;

}

