/*
 * linux/drivers/industrialio/imu/adis16350.h
 * Copyright (C) 2008 Jonathan Cameron
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#define ADIS16350_ENDURANCE_LSB_ADDR		0x01
/* Data output registers - read from either msb or lsb will get both.*/
#define ADIS16350_POWER_SUPPLY_ADDR		0x02
#define ADIS16350_X_GYRO_ADDR			0x04
#define ADIS16350_Y_GYRO_ADDR			0x06
#define ADIS16350_Z_GYRO_ADDR			0x08
#define ADIS16350_X_ACCEL_ADDR			0x0A
#define ADIS16350_Y_ACCEL_ADDR			0x0C
#define ADIS16350_Z_ACCEL_ADDR			0x0E
/* Temperatures are from sensors on each gyro */
#define ADIS16350_X_TEMP_ADDR			0x10
#define ADIS16350_Y_TEMP_ADDR			0x12
#define ADIS16350_Z_TEMP_ADDR			0x14
#define ADIS16350_ADC_ADDR			0x16

/* Device bias offset registers */
#define ADIS16350_X_GYRO_OFF_LSB_ADDR		0x1A
#define ADIS16350_Y_GYRO_OFF_LSB_ADDR		0x1C
#define ADIS16350_Z_GYRO_OFF_LSB_ADDR		0x1E
#define ADIS16350_X_ACCEL_OFF_LSB_ADDR		0x20
#define ADIS16350_Y_ACCEL_OFF_LSB_ADDR		0x22
#define ADIS16350_Z_ACCEL_OFF_LSB_ADDR		0x24

/* Alarm amplitude thresholds */
#define ADIS16350_ALM_1_AMP_THRESH_LSB_ADDR	0x26
#define ADIS16350_ALM_1_AMP_THRESH_MSB_ADDR	0x27
#define ADIS16350_ALM_2_AMP_THRESH_LSB_ADDR	0x28
#define ADIS16350_ALM_2_AMP_THRESH_MSB_ADDR	0x29
#define ADIS16350_ALM_SAMP_THRESH_ADDR(a)		\
  ((a) == 1 ? ADIS16350_ALM_1_AMP_THRESH_LSB_ADDR	\
   : ADIS16350_ALM_2_AMP_THRESH_LSB_ADDR)

/* Alarm sample sizes - only lsb's used */
#define ADIS16350_ALM_1_SAMP_SIZE_LSB_ADDR	0x2A
#define ADIS16350_ALM_2_SAMP_SIZE_LSB_ADDR	0x2C
#define ADIS16350_ALM_SAMP_SIZE_ADDR(a)			\
  ((a) == 1 ? ADIS16350_ALM_1_SAMP_SIZE_LSB_ADDR	\
   : ADIS16350_ALM_2_SAMP_SIZE_LSB_ADDR)

/* Alarm control register */
#define ADIS16350_ALM_CTRL_LSB_ADDR		0x2E
#define ADIS16350_ALM_CTRL_MSB_ADDR		0x2F

/* Auxilliary DAC data (12 bit)*/
#define ADIS16350_DAC_LSB_ADDR			0x30

/* Miscellaneous control register */
#define ADIS16350_MSC_CTRL_LSB_ADDR		0x34
#define ADIS16350_MSC_CTRL_MSB_ADDR		0x35

/* Internal sample period ctrl register */
#define ADIS16350_SAMP_PERIOD_LSB_ADDR		0x36

/* Dynamic range / digital filter control register */
#define ADIS16350_FILTERING_ADDR		0x38
#define ADIS16350_SENSITIVITY_ADDR		0x39

/* Status register */
#define ADIS16350_STATUS_LSB_ADDR		0x3C

/* System command register (MSB never used)*/
#define ADIS16350_COMMAND_LSB_ADDR		0x3E


#define ADIS16350_COMMAND_SOFTWARE_RESET	(1 << 7)
#define ADIS16350_COMMAND_PREC_AUTO_NULL	(1 << 4)
#define ADIS16350_COMMAND_FLASH_UPDATE		(1 << 3)
/* This actually enables the DAC settings previously written
 * to the DAC data registers.
 */
#define ADIS16350_COMMAND_DAC_DATA_LATCH	(1 << 2)
#define ADIS16350_COMMAND_FACTORY_RESTORE	(1 << 1)
#define ADIS16350_COMMAND_AUTO_NULL		(1 << 0)


#define ADIS16350_MSC_CTRL_MSB_INT_SELF_TEST_EN		(1 << 2)
#define ADIS16350_MSC_CTRL_MSB_MAN_SELF_TEST_NEG_STIM	(1 << 1)
#define ADIS16350_MSC_CTRL_MSB_MAN_SELF_TEST_POS_STIM	(1 << 0)

/* lsb elements */
/* Enable bias compensationfor the effects of linear acceleration
 * on the gyroscopes
 */
#define ADIS16350_MSC_CTRL_LSB_GYRO_COMP_FOR_ACCEL	(1 << 7)
#define ADIS16350_MSC_CTRL_LSB_ACCEL_ORIGIN_ALIGN	(1 << 6)
#define ADIS16350_MSC_CTRL_LSB_DATA_RDY_ENABLE		(1 << 2)
#define ADIS16350_MSC_CTRL_LSB_DATA_RDY_ACT_HIGH	(1 << 1)
#define ADIS16350_MSC_CTRL_LSB_DATA_RDY_LINE_2		(1 << 0)

/* As status is read only it is only ever read in one go */
#define ADIS16350_STATUS_Z_ACCEL_SELFTEST_FAIL		(1 << 15)
#define ADIS16350_STATUS_Y_ACCEL_SELFTEST_FAIL		(1 << 14)
#define ADIS16350_STATUS_X_ACCEL_SELFTEST_FAIL		(1 << 13)
#define ADIS16350_STATUS_Z_GYRO_SELFTEST_FAIL		(1 << 12)
#define ADIS16350_STATUS_Y_GYRO_SELFTEST_FAIL		(1 << 11)
#define ADIS16350_STATUS_X_GYRO_SELFTESTT_FAIL		(1 << 10)
#define ADIS16350_STATUS_ALM_2				(1 << 9)
#define ADIS16350_STATUS_ALM_1				(1 << 8)

#define ADIS16350_STATUS_SELFTEST_ERROR			(1 << 5)
#define ADIS16350_STATUS_OVER_RANGE			(1 << 4)
#define ADIS16350_STATUS_SPI_FAIL			(1 << 3)
#define ADIS16350_STATUS_CTRL_REG_UPDATE_FAIL		(1 << 2)
#define ADIS16350_STATUS_POWER_HIGH			(1 << 1)
#define ADIS16350_STATUS_POWER_LOW			(1 << 0)

#define ADIS16350_ALM_AMP_THRESH_MSB_GREATER		(1 << 7)
#define ADIS16350_ALM_AMP_THRESH_MSB_LESS		0
/* N.B. 14 bits used for value */

/* These all go in either the top 4 bits for alarm 2 or the
 * bottom 4 bits for alarm 1 of ALM_CTRL_MSB_ADDR
 */
/* use the top bit to signal greater than or less that */
#define ADIS16350_ALM_GT(a) ((a) | (1 << 7))
#define ADIS16350_ALM_LT(a) (a)
#define ADIS16350_ALM_SEL_DISABLE		0x0
#define ADIS16350_ALM_SEL_POWER			0x1
#define ADIS16350_ALM_SEL_X_GYRO		0x2
#define ADIS16350_ALM_SEL_Y_GYRO		0x3
#define ADIS16350_ALM_SEL_Z_GYRO		0x4
#define ADIS16350_ALM_SEL_X_ACCEL		0x5
#define ADIS16350_ALM_SEL_Y_ACCEL		0x6
#define ADIS16350_ALM_SEL_Z_ACCEL		0x7
/* Correspond to temp outputs of different gyros */
#define ADIS16350_ALM_SEL_X_TEMP		0x8
#define ADIS16350_ALM_SEL_Y_TEMP		0x9
#define ADIS16350_ALM_SEL_Z_TEMP		0xA
#define ADIS16350_ALM_SEL_ADC			0xB

#define ADIS16350_ALM_SEL_MASK 0xF

#define ADIS16350_ALM_CTRL_SEL_TO_ALM_NO(_num, _sel)	\
	(_num == 2) ? _sel << 4 : _sel

#define ADIS16350_ALM_CTRL_LSB_ROC(_num) (1 << (5 + (_num)))
#define ADIS16350_ALM_CTRL_LSB_USE_FILTERED	(1 << 4)

#define ADIS16350_ALM_CTRL_LSB_ENABLE		(1 << 2)
#define ADIS16350_ALM_CTRL_LSB_ACT_HIGH		(1 << 1)
#define ADIS16350_ALM_CTRL_LSB_ALM_LINE_2	(1 << 0)
#define ADIS16350_ALM_CTRL_LSB_ALM_LINE_1	(0)

#define ADIS16350_SENSITIVITY_300_DPS 0x04
#define ADIS16350_SENSITIVITY_150_DPS 0x02
#define ADIS16350_SENSITIVITY_75_DPS 0x01

#define ADIS16350_WRITE(a) ((1 << 7) | (a))
#define ADIS16350_READ(a) (a)

struct adis16350_alarm_info {
	unsigned mask : 4;
	unsigned pol :1;
	unsigned roc :1;
	int event_code;
};
/* In header as effectively a set of defines */
static const struct adis16350_alarm_info adis16350_alarms[] = {
	{},
	{ADIS16350_ALM_SEL_X_ACCEL, 1, 0, IIO_EVENT_CODE_ACCEL_X_HIGH},
	{ADIS16350_ALM_SEL_X_ACCEL, 0, 0, IIO_EVENT_CODE_ACCEL_X_LOW},
	{ADIS16350_ALM_SEL_Y_ACCEL, 1, 0, IIO_EVENT_CODE_ACCEL_Y_HIGH},
	{ADIS16350_ALM_SEL_Y_ACCEL, 0, 0, IIO_EVENT_CODE_ACCEL_Y_LOW},
	{ADIS16350_ALM_SEL_Z_ACCEL, 1, 0, IIO_EVENT_CODE_ACCEL_Z_HIGH},
	{ADIS16350_ALM_SEL_Z_ACCEL, 0, 0, IIO_EVENT_CODE_ACCEL_Z_LOW},
	{ADIS16350_ALM_SEL_X_GYRO, 1, 0, IIO_EVENT_CODE_GYRO_X_HIGH},
	{ADIS16350_ALM_SEL_X_GYRO, 0, 0, IIO_EVENT_CODE_GYRO_X_LOW},
	{ADIS16350_ALM_SEL_Y_GYRO, 1, 0, IIO_EVENT_CODE_GYRO_Y_HIGH},
	{ADIS16350_ALM_SEL_Y_GYRO, 0, 0, IIO_EVENT_CODE_GYRO_Y_LOW},
	{ADIS16350_ALM_SEL_Z_GYRO, 1, 0, IIO_EVENT_CODE_GYRO_Z_HIGH},
	{ADIS16350_ALM_SEL_Z_GYRO, 0, 0, IIO_EVENT_CODE_GYRO_Z_LOW},
	{ADIS16350_ALM_SEL_X_ACCEL, 1, 1, IIO_EVENT_CODE_ACCEL_X_ROC_HIGH},
	{ADIS16350_ALM_SEL_X_ACCEL, 0, 1, IIO_EVENT_CODE_ACCEL_X_ROC_LOW},
	{ADIS16350_ALM_SEL_Y_ACCEL, 1, 1, IIO_EVENT_CODE_ACCEL_Y_ROC_HIGH},
	{ADIS16350_ALM_SEL_Y_ACCEL, 0, 1, IIO_EVENT_CODE_ACCEL_Y_ROC_LOW},
	{ADIS16350_ALM_SEL_Z_ACCEL, 1, 1, IIO_EVENT_CODE_ACCEL_Z_ROC_HIGH},
	{ADIS16350_ALM_SEL_Z_ACCEL, 0, 1, IIO_EVENT_CODE_ACCEL_Z_ROC_LOW},
	{ADIS16350_ALM_SEL_X_GYRO, 1, 1, IIO_EVENT_CODE_GYRO_X_ROC_HIGH},
	{ADIS16350_ALM_SEL_X_GYRO, 0, 1, IIO_EVENT_CODE_GYRO_X_ROC_LOW},
	{ADIS16350_ALM_SEL_Y_GYRO, 1, 1, IIO_EVENT_CODE_GYRO_Y_ROC_HIGH},
	{ADIS16350_ALM_SEL_Y_GYRO, 0, 1, IIO_EVENT_CODE_GYRO_Y_ROC_LOW},
	{ADIS16350_ALM_SEL_Z_GYRO, 1, 1, IIO_EVENT_CODE_GYRO_Z_ROC_HIGH},
	{ADIS16350_ALM_SEL_Z_GYRO, 0, 1, IIO_EVENT_CODE_GYRO_Z_ROC_LOW},
};

enum adis16350_alarm_mode {
	adis16350_alm_disabled = 0,
	adis16350_alm_accel_x_gt_static,
	adis16350_alm_accel_x_lt_static,
	adis16350_alm_accel_y_gt_static,
	adis16350_alm_accel_y_lt_static,
	adis16350_alm_accel_z_gt_static,
	adis16350_alm_accel_z_lt_static,

	adis16350_alm_gyro_x_gt_static,
	adis16350_alm_gyro_x_lt_static,
	adis16350_alm_gyro_y_gt_static,
	adis16350_alm_gyro_y_lt_static,
	adis16350_alm_gyro_z_gt_static,
	adis16350_alm_gyro_z_lt_static,

	adis16350_alm_accel_x_gt_roc,
	adis16350_alm_accel_x_lt_roc,
	adis16350_alm_accel_y_gt_roc,
	adis16350_alm_accel_y_lt_roc,
	adis16350_alm_accel_z_gt_roc,
	adis16350_alm_accel_z_lt_roc,

	adis16350_alm_gyro_x_gt_roc,
	adis16350_alm_gyro_x_lt_roc,
	adis16350_alm_gyro_y_gt_roc,
	adis16350_alm_gyro_y_lt_roc,
	adis16350_alm_gyro_z_gt_roc,
	adis16350_alm_gyro_z_lt_roc,
};

struct adis16350_state {
	struct spi_device *spi;
	struct iio_dev *indio_dev;
	u32 max_speed_hz;
	s64 last_timestamp_datardy;
	s64 last_timestamp_event;
	struct adis16350_platform_data *pdata;
	struct work_struct work_data_rdy_event;
	struct work_struct work_data_rdy_ring;

	struct iio_work_cont work_alarm_event;

	u8 next_alarm;

	enum adis16350_alarm_mode current_alms[2];
	/* alarm related */
	s16 alarm_val[ARRAY_SIZE(adis16350_alarms)];
	u8 alarm_samp[ARRAY_SIZE(adis16350_alarms)];
};

#define ADIS16350_BASE_610MICROSECS 0
#define ADIS16350_BASE_18921MICROSECS 1
struct adis16350_sampling_freq {
	const char *name;
	unsigned int timebase :1;
	unsigned int multiplier :6;
};
