/* The industrial I/O periodic timer registration code
 *
 * Copyright (c) 2008 Jonathan Cameron
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 *
 */
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/list.h>
#include <linux/mutex.h>
#include <linux/slab.h>
#include <linux/module.h>
#include <linux/industrialio_ptimer.h>

LIST_HEAD(industrialio_ptimer_board_info_list);
EXPORT_SYMBOL_GPL(industrialio_ptimer_board_info_list);

DEFINE_MUTEX(industrialio_ptimer_board_lock);
EXPORT_SYMBOL_GPL(industrialio_ptimer_board_lock);


int __init
industrialio_register_ptimer(struct ptimer_info const *info, unsigned n)
{
	int i;
	struct ptimer_info_listel *pi;

	mutex_lock(&industrialio_ptimer_board_lock);
	for (i = 0; i < n; i++) {
		pi = kzalloc(sizeof(*pi), GFP_KERNEL);
		if (!pi) {
			mutex_unlock(&industrialio_ptimer_board_lock);
			return -ENOMEM;
		}
		strncpy(pi->info.name, info[i].name, IIO_PTIMER_NAME_SIZE);
		list_add_tail(&pi->list, &industrialio_ptimer_board_info_list);
	}
	mutex_unlock(&industrialio_ptimer_board_lock);

	return 0;
}
