#include <linux/module.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/i2c.h>

#include "superbird2.h"

MODULE_AUTHOR("Brian Mayton");
MODULE_DESCRIPTION("Driver for the CSE466 SuperBird board");
MODULE_VERSION("2.0");
MODULE_LICENSE("GPL");

static int superbird_probe(struct i2c_client *client, const struct i2c_device_id *id);
static int superbird_remove(struct i2c_client *client);
static int superbird_detect(struct i2c_client *client, int kind, struct i2c_board_info *board);

static unsigned short normal_i2c[] = { SUPERBIRD_I2C_ADDR, I2C_CLIENT_END };
I2C_CLIENT_INSMOD;

static struct i2c_device_id superbird_id_table[] = {
	{ "superbird", 0 },
	{ }
};

MODULE_DEVICE_TABLE(i2c, superbird_id_table);

static struct i2c_driver superbird_driver = {
	.driver = {
		.name = "superbird",
		.owner = THIS_MODULE,
	},
	
	.id_table = superbird_id_table,
	.probe = superbird_probe,
	.remove = __devexit_p(superbird_remove),
	.address_data = &addr_data,
	.detect = superbird_detect,
};

static int superbird_detect(struct i2c_client *client, int kind, struct i2c_board_info *board) {
	printk(KERN_INFO "Detect called for %d.\n", board->addr);
	return -ENODEV;
}

static int superbird_probe(struct i2c_client *client, const struct i2c_device_id *id) {
	printk(KERN_INFO "Probe called.\n");
	return -ENODEV;
};

static int __devexit superbird_remove(struct i2c_client *client) {
	printk(KERN_INFO "Remove called.\n");
	return 0;
};

int superbird_read_reg(struct i2c_client *client, uint8_t reg) {
	uint8_t data;
	i2c_master_send(client, &reg, 1);
	i2c_master_recv(client, &data, 1);
	return data;
}

int superbird_write_reg(struct i2c_client *client, uint8_t reg, uint8_t data) {
	char buf[2];
	buf[0] = reg;
	buf[1] = data;
	return i2c_master_send(client, buf, 2);
}

static int __init superbird_init(void) {
	printk(KERN_INFO "Loading SuperBird module 2.\n");
	return 0;
	//return i2c_add_driver(&superbird_driver);
}

static void __exit superbird_exit(void) {
	printk(KERN_INFO "Unloading SuperBird module.\n");
	//i2c_del_driver(&superbird_driver);
}

module_init(superbird_init);
module_exit(superbird_exit);