

#ifndef _IIO_RING_SW_H_
#define _IIO_RING_SW_H_
/* NEEDS COMMENTS */
/* The intention is that this should be a separate module from the iio core.
 * This is a bit like supporting algorithms dependent on what the device
 * driver requests - some may support multiple options */

/* Should this be here? directory wise that is*/
#include <linux/autoconf.h>
#include <linux/industrialio/iio.h>
#include <linux/industrialio/ring_generic.h>

#if defined CONFIG_IIO_SW_RING || defined CONFIG_IIO_SW_RING_MODULE

int iio_create_sw_rb(void **r);
int iio_init_sw_rb(void *r, void *indio_dev);
void iio_exit_sw_rb(void *r);
void iio_free_sw_rb(void *r);
void iio_mark_sw_rb_in_use(void *r);
void iio_unmark_sw_rb_in_use(void *r);
int iio_read_last_from_sw_rb(void *r, u8 *data);
int iio_store_to_sw_rb(void *rb, u8 *data, s64 timestamp);
int iio_rip_sw_rb(void *r, size_t count, u8 **data, int *dead_offset);

/* Parameter access functions */
int iio_sw_rb_get_bytes_per_datum(void *r);
int iio_sw_rb_set_bytes_per_datum(void *r);

int iio_request_update_sw_rb(void *r);
int iio_mark_update_needed_sw_rb(void *r);

/* These are actually more general that their names suggest */
int iio_get_bpd_sw_rb(void *r);
int iio_set_bpd_sw_rb(void *r, int bpd);

int iio_get_length_sw_rb(void *r);
int iio_set_length_sw_rb(void *r, int length);

static inline void iio_ring_sw_register_funcs(struct iio_ring_access_funcs *ra)
{
	ra->_create = &iio_create_sw_rb;
	ra->_free = &iio_free_sw_rb;
	ra->_init = &iio_init_sw_rb;
	ra->_exit = &iio_exit_sw_rb;

	ra->mark_in_use = &iio_mark_sw_rb_in_use;
	ra->unmark_in_use = &iio_unmark_sw_rb_in_use;

	ra->store_to = &iio_store_to_sw_rb;
	ra->read_last = &iio_read_last_from_sw_rb;
	ra->rip_lots = &iio_rip_sw_rb;

	ra->mark_param_change = &iio_mark_update_needed_sw_rb;
	ra->request_update = &iio_request_update_sw_rb;

	ra->get_bpd = &iio_get_bpd_sw_rb;
	ra->set_bpd = &iio_set_bpd_sw_rb;

	ra->get_length = &iio_get_length_sw_rb;
	ra->set_length = &iio_set_length_sw_rb;
};
//int iio_request_update_sw_rb(void *ring);


/**
 * struct iio_sw_ring_buffer - software ring buffer
 * @buf:		generic ring buffer elements
 * @data:		the ring buffer memory
 * @read_p:		read pointer (oldest available)
 * @write_p:		write pointer
 * @last_written_p:	read pointer (newest available)
 * @half_p:		half buffer length behind write_p (event generation)
 * @use_count:		reference count to prevent resizing when in use
 * @update_needed:	flag to indicated change in size requested
 * @use_lock:		lock to prevent change in size when in use
 */
/*  first element of all ring buffers must be a struct iio_ring_buffer!*/
struct iio_sw_ring_buffer {
	struct iio_ring_buffer buf;
	unsigned char		*data;
	unsigned char		*read_p;
	unsigned char		*write_p;
	unsigned char		*last_written_p;
	/* used to act as a point at which to signal an event */
	unsigned char		*half_p;
	int			use_count;
	int			update_needed;
	spinlock_t		use_lock;
};


//why here?


#else /* CONFIG_IIO_RING_BUFFER*/
static inline void iio_ring_sw_register_funcs(struct iio_ring_access_funcs *ra) {};
#endif

#endif /* _IIO_RING_SW_H_ */
