/*
 * TOS-MAC user interface
 * Driver is registered as a character device,
 * so normal system calls(open/close/read/write/ioctl)
 * are used to recv/trans data and configure radio device.
 */

#ifndef __TOSMAC_H
#define __TOSMAC_H
#include <linux/types.h>

#define TOSMAC_MAJOR	240
#define TOSMAC_DEVICE   "/dev/tosmac"

/*default vlaue of data lengh in packet*/
#define TOSH_DATA_LENGTH  28
/*
 * Packet structure
 * To be compatable with TinyOS, we provide a packet data structure,
 * which is the definition of packet used in TinyOS.
 * User program need to set up a packet before making system calls
 */
/* This packet definition may be changed as developing going on */
typedef struct __TOS_Msg
{
	__u8 length;    // data length include header and footer
	__u8 fcfhi;
	__u8 fcflo;
	__u8 dsn;
	__u16 destpan;  // destPAN
	__u16 addr;	   // destAddr
	__u8 type;
	__u8 group;
	__s8 data[TOSH_DATA_LENGTH];
} TOS_Msg;

#define MSG_ERROR_0	-1
#define MSG_ERROR_1	-2

/*
 * Important:
 * TOSMAC driver is a non-standard character device.
 * Please read descriptions below for each function for details.
 */

/*
 * Open device
 * Use open( ) system call
 * Device can be opened in blocking or non-blocking mode
 * flags supported:
 *	O_RDWR			// read/write in blocking mode
 *	O_RDWR | O_NONBLOCK	// read/write in non-blocking mode
 */
//int open(TOSMAC_DEVICE *device_path, int flags);

/*
 * Close device
 * Use close() system call
 */
//int close(int fd);

/*
 * Read data
 * Use read( ) system call.
 * Each time, one packet will be returned if there are data received.
 * void *buf is the pointer to a packet.
 * count should always be sizeof(TOS_Msg)
 * If user give a count value not equal with sizeof(TOS_Msg), then an error code
 * will return.
 * MSG_ERR_0: count is shorter than sizeof(TOS_Msg)
 * MSG_ERR_1: count is longer than sizeof(TOS_Msg)
 */
//ssize_t read(int fd, void *buf, size_t count);

/*
 * Write data
 * Use write( ) system call.
 * Each time, one packet will be sent. Do init of packet before sending.
 * So buf is the pointer to a packet.
 * count should always be sizeof(TOS_Msg)
 * If user give a count value not equal with sizeof(TOS_Msg), then an error code
 * will return.
 * MSG_ERR_0: count is shorter than sizeof(TOS_Msg)
 * MSG_ERR_1: count is longer than sizeof(TOS_Msg)
 */
//ssize_t write(int fd, const void *buf, size_t count);

/*
 * Poll device
 * Use poll() or select() system call to poll device.
 */
// int select(int n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds,
//            struct timeval *timeout);
// int poll(struct pollfd *ufds, nfds_t nfds, int timeout);

/*
 * Configure device
 * Use ioctl() system call.
 * cmd: the feature needs to be set, such as channel, power
 * arg: the value will be set to that feature if necessary
 */
//int ioctl(int fd, unsigned int cmd, unsigned int arg);

/*IOMAGIC for TOSMAC driver*/
#define TOSMAC_MAGIC   0xF4
/*Commands for IOCTL, more commands will be added*/
// set radio channel
#define TOSMAC_SET_CHANNEL    _IOW(TOSMAC_MAGIC, 1, __u16)
// switch receiver on/off
#define TOSMAC_SET_TRANS_ONOFF  _IO (TOSMAC_MAGIC, 2, __u8)
// change default data length
#define TOSMAC_SET_DATALENGTH _IOW(TOSMAC_MAGIC, 3, __u16)

#endif
