/*
 * superbird.c  --  SoC audio for SuperBird
 *
 * Copyright 2008 University of Washington CSE
 *
 * Author: Brian Mayton <bmayton@cs.washington.edu>
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation; version 2 ONLY.
 *
 */

#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/device.h>

#include <sound/core.h>
#include <sound/pcm.h>
#include <sound/soc.h>
#include <sound/soc-dapm.h>

#include <asm/mach-types.h>
#include <mach/pxa-regs.h>
#include <mach/hardware.h>
#include <mach/audio.h>

extern int superbird_audio_pwr_en(int enable);

#include "../codecs/ak4642.h"
#include "pxa2xx-pcm.h"
#include "pxa2xx-i2s.h"

static struct snd_soc_machine superbird;

static int superbird_hw_params(struct snd_pcm_substream *substream,
	struct snd_pcm_hw_params *params)
{
	struct snd_soc_pcm_runtime *rtd = substream->private_data;
	struct snd_soc_dai *codec_dai = rtd->dai->codec_dai;
	struct snd_soc_dai *cpu_dai = rtd->dai->cpu_dai;
	int ret = 0;

	/* set codec DAI configuration */
	ret = codec_dai->dai_ops.set_fmt(codec_dai, SND_SOC_DAIFMT_I2S |
		SND_SOC_DAIFMT_NB_NF | SND_SOC_DAIFMT_CBS_CFS);
	if (ret < 0)
		return ret;

	/* set cpu DAI configuration */
	ret = cpu_dai->dai_ops.set_fmt(cpu_dai, SND_SOC_DAIFMT_I2S |
		SND_SOC_DAIFMT_NB_NF | SND_SOC_DAIFMT_CBS_CFS);
	if (ret < 0)
		return ret;

	/* set the I2S system clock as input (unused) */
	ret = cpu_dai->dai_ops.set_sysclk(cpu_dai, PXA2XX_I2S_SYSCLK, 0,
		SND_SOC_CLOCK_OUT);
	if (ret < 0)
		return ret;

	return 0;
}

static struct snd_soc_ops superbird_ops = {
	.hw_params = superbird_hw_params,
};

static int superbird_ak4642_init(struct snd_soc_codec *codec) {
	printk(KERN_DEBUG "superbird_ak4642_init\n");
	snd_soc_dapm_sync(codec);
	return 0;
}

static struct snd_soc_dai_link superbird_dai[] = {
{
	.name = "SuperBird Audio",
	.stream_name = "SuperBird Audio",
	.cpu_dai = &pxa_i2s_dai,
	.codec_dai = &ak4642_dai,
	.init = superbird_ak4642_init,
	.ops = &superbird_ops,
},
};

static struct snd_soc_machine superbird = {
	.name = "SuperBird",
	.dai_link = superbird_dai,
	.num_links = ARRAY_SIZE(superbird_dai),
};

static struct ak4642_setup_data superbird_ak4642_setup = {
	.i2c_address = 0x12,
};


static struct snd_soc_device superbird_snd_devdata = {
	.machine = &superbird,
	.platform = &pxa2xx_soc_platform,
	.codec_dev = &soc_codec_dev_ak4642,
    .codec_data = &superbird_ak4642_setup,
};

static struct platform_device *superbird_snd_device;

static int __init superbird_init(void)
{
	int ret;

    superbird_audio_pwr_en(1);

	superbird_snd_device = platform_device_alloc("soc-audio", -1);
	if (!superbird_snd_device)
		return -ENOMEM;

	platform_set_drvdata(superbird_snd_device, &superbird_snd_devdata);
	superbird_snd_devdata.dev = &superbird_snd_device->dev;
	ret = platform_device_add(superbird_snd_device);

	if (ret)
		platform_device_put(superbird_snd_device);

	return ret;
}

static void __exit superbird_exit(void)
{
    superbird_audio_pwr_en(0);
	platform_device_unregister(superbird_snd_device);
}

module_init(superbird_init);
module_exit(superbird_exit);

/* Module information */
MODULE_AUTHOR("Brian Mayton");
MODULE_DESCRIPTION("ALSA SoC driver for SuperBird");
MODULE_LICENSE("GPL");
MODULE_VERSION("1.0.0");


